package pps.core.common.utils;


import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import xstartup.base.util.XDateUtils;

import java.io.IOException;
import java.io.InputStream;
import java.text.DecimalFormat;
import java.util.*;
import java.util.logging.Logger;

/**
 * Author: lixueyan
 * Date: 2021-01-01
 * Time: 11:09
 * Description: 生成Excel并写入数据
 */
public class ExcelUtils {
    private static final String XLS = "xls";
    private static final String XLSX = "xlsx";
    private static Logger logger = Logger.getLogger(ExcelUtils.class.getName()); // 日志打印类

    /**
     * 生成Excel并写入数据信息
     *
     * @param dataList 数据列表
     * @return 写入数据后的工作簿对象
     */
    public static Workbook exportData(List<Map<String, Object>> dataList, List<String> headerList) {
        // 生成xlsx的Excel
        Workbook workbook = new SXSSFWorkbook();
        CellStyle cs = workbook.createCellStyle();
        cs.setWrapText(true); //内容自动换行
        cs.setAlignment(HorizontalAlignment.CENTER); //水平居中
        cs.setVerticalAlignment(VerticalAlignment.CENTER);//垂直居中
        // 生成Sheet表，写入第一行的列头
        Sheet sheet = buildDataSheet(workbook, headerList);
        //构建每行的数据内容
        int rowNum = 1;
        for (Map<String, Object> data : dataList) {
            if (CollUtil.isEmpty(data)) {
                continue;
            }
            //输出行数据
            Row row = sheet.createRow(rowNum++);
            convertDataToRow(data, row, headerList, cs);
        }
        return workbook;
    }

    /**
     * 生成sheet表，并写入第一行数据（列头）
     *
     * @param workbook   工作簿对象
     * @param headerList
     * @return 已经写入列头的Sheet
     */
    private static Sheet buildDataSheet(Workbook workbook, List<String> headerList) {
        Sheet sheet = workbook.createSheet();
        // 设置列头宽度
        for (int i = 0; i < headerList.size(); i++) {
            sheet.setColumnWidth(i, 4000);
        }
        // 设置默认行高
        sheet.setDefaultRowHeight((short) 400);
        // 构建头单元格样式
        CellStyle cellStyle = buildHeadCellStyle(sheet.getWorkbook());
        // 写入第一行各列的数据
        Row head = sheet.createRow(0);
        for (int i = 0; i < headerList.size(); i++) {
            Cell cell = head.createCell(i);
            cell.setCellValue(headerList.get(i));
            cell.setCellStyle(cellStyle);
        }
        return sheet;
    }

    /**
     * 设置第一行列头的样式
     *
     * @param workbook 工作簿对象
     * @return 单元格样式对象
     */
    private static CellStyle buildHeadCellStyle(Workbook workbook) {
        CellStyle style = workbook.createCellStyle();
        //对齐方式设置
        style.setAlignment(HorizontalAlignment.CENTER);
        //边框颜色和宽度设置
        style.setBorderBottom(BorderStyle.THIN);
        style.setBottomBorderColor(IndexedColors.BLACK.getIndex()); // 下边框
        style.setBorderLeft(BorderStyle.THIN);
        style.setLeftBorderColor(IndexedColors.BLACK.getIndex()); // 左边框
        style.setBorderRight(BorderStyle.THIN);
        style.setRightBorderColor(IndexedColors.BLACK.getIndex()); // 右边框
        style.setBorderTop(BorderStyle.THIN);
        style.setTopBorderColor(IndexedColors.BLACK.getIndex()); // 上边框
        //设置背景颜色
        style.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
        style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        //粗体字设置
        Font font = workbook.createFont();
        font.setBold(true);
        style.setFont(font);
        return style;
    }

    /**
     * 将数据转换成行
     *
     * @param data       源数据
     * @param row        行对象
     * @param headerList
     * @return
     */
    private static void convertDataToRow(Map<String, Object> data, Row row, List<String> headerList, CellStyle cs) {
        int cellNum = 0;
        Cell cell;
        for (int i = 0; i < headerList.size(); i++) {
            Object val = data.get(headerList.get(i));
            cell = row.createCell(cellNum++);
            cell.setCellValue(null == val ? "" : val + "");
            cell.setCellStyle(cs);
        }
    }

    /**
     * 根据文件后缀名类型获取对应的工作簿对象
     *
     * @param inputStream 读取文件的输入流
     * @param fileType    文件后缀名类型（xls或xlsx）
     * @return 包含文件数据的工作簿对象
     * @throws IOException
     */
    public static Workbook getWorkbook(InputStream inputStream, String fileType) throws IOException {
        Workbook workbook = null;
        if (CharSequenceUtil.equalsIgnoreCase(fileType, XLS)) {
            workbook = new HSSFWorkbook(inputStream);
        } else if (CharSequenceUtil.equalsIgnoreCase(fileType, XLSX)) {
            workbook = new XSSFWorkbook(inputStream);
        }
        return workbook;
    }

    /**
     * 读取Excel文件内容
     *
     * @param fileName   要读取的Excel文件所在路径
     * @param sheetIndex 要读取的指定sheet页，为空时读取全部
     * @return 读取结果列表，读取失败时返回null
     */
    public static List<Map<String, Object>> readExcel(InputStream inputStream, String fileName, List<String> headerList, Integer sheetIndex) {
        Workbook workbook;
        try {
            // 获取Excel后缀名
            String fileType = fileName.substring(fileName.lastIndexOf(".") + 1);
            // 获取Excel工作簿
            workbook = getWorkbook(inputStream, fileType);
            // 读取excel中的数据
            return parseExcel(workbook, headerList, sheetIndex);
        } catch (Exception e) {
            logger.warning("解析Excel失败，文件名：" + fileName + " 错误信息：" + e.getMessage());
            return null;
        } finally {
            try {
                if (null != inputStream) {
                    inputStream.close();
                }
            } catch (Exception e) {
                logger.warning("关闭数据流出错！错误信息：" + e.getMessage());
            }
        }
    }

    /**
     * 解析Excel数据
     *
     * @param workbook   Excel工作簿对象
     * @param headerList
     * @param sheetIndex
     * @return 解析结果
     */
    public static List<Map<String, Object>> parseExcel(Workbook workbook, List<String> headerList, Integer sheetIndex) {
        List<Map<String, Object>> resultDataList = new ArrayList<>();
        // 解析sheet
        for (int sheetNum = 0; sheetNum < workbook.getNumberOfSheets(); sheetNum++) {
            if (ObjectUtil.isNotNull(sheetIndex) && !sheetIndex.equals(sheetNum)) {
                continue;
            }
            Sheet sheet = workbook.getSheetAt(sheetNum);
            // 校验sheet是否合法
            if (ObjectUtil.isNull(sheet)) {
                continue;
            }
            // 获取第一行数据
            int firstRowNum = sheet.getFirstRowNum();
            Row firstRow = sheet.getRow(firstRowNum);
            if (ObjectUtil.isNull(firstRow)) {
                logger.warning("解析Excel失败，在第一行没有读取到任何数据！");
            }
            // 解析每一行的数据，构造数据对象
            int rowStart = firstRowNum + 1;
            int rowEnd = sheet.getPhysicalNumberOfRows();
            for (int rowNum = rowStart; rowNum < rowEnd; rowNum++) {
                Row row = sheet.getRow(rowNum);
                if (ObjectUtil.isNull(row)) {
                    continue;
                }
                Map<String, Object> resultData = convertRowToData(row, headerList);
                if (CollUtil.isEmpty(resultData)) {
                    logger.warning("第 " + row.getRowNum() + "行数据不合法，已忽略！");
                    continue;
                }
                resultDataList.add(resultData);
            }
        }
        return resultDataList;
    }

    /**
     * 将单元格内容转换为字符串
     *
     * @param cell
     * @return
     */
    private static String convertCellValueToString(Cell cell) {
        if (ObjectUtil.isNull(cell)) {
            return null;
        }
        String returnValue = null;
        switch (cell.getCellType()) {
            case NUMERIC:   //数字
                if (DateUtil.isCellDateFormatted(cell)) {
                    Date d = cell.getDateCellValue();
                    returnValue = XDateUtils.getDateTimeString(d);
                } else {
                    Double doubleValue = cell.getNumericCellValue();
                    // 格式化科学计数法，取一位整数
                    DecimalFormat df = new DecimalFormat("0.00000");
                    returnValue = df.format(doubleValue);
                }
                break;
            case STRING:    //字符串
                returnValue = cell.getStringCellValue();
                break;
            case BOOLEAN:   //布尔
                boolean booleanValue = cell.getBooleanCellValue();
                returnValue = Boolean.toString(booleanValue);
                break;
            case BLANK:     // 空值
                break;
            case FORMULA:   // 公式
                returnValue = cell.getCellFormula();
                break;
            case ERROR:     // 故障
                break;
            default:
                break;
        }
        return returnValue;
    }

    /**
     * 提取每一行中需要的数据，构造成为一个结果数据对象
     * <p>
     * 当该行中有单元格的数据为空或不合法时，忽略该行的数据
     *
     * @param row        行数据
     * @param headerList
     * @return 解析后的行数据对象，行数据错误时返回null
     */
    private static Map<String, Object> convertRowToData(Row row, List<String> headerList) {
        Map<String, Object> resultData = new HashMap<>();
        Cell cell;
        int cellNum = 0;
        for (String s : headerList) {
            cell = row.getCell(cellNum++);
            String name = convertCellValueToString(cell);
            resultData.put(s, name);
        }
        return resultData;
    }
}